/* -*- mode: c++; c-basic-offset: 3; -*- */

#ifndef GDSPLOT_PARS_HH
#define GDSPLOT_PARS_HH

#include "tri_state.hh"
#include <iosfwd>
#include <string>

class GDSPlot;

class plot_pars {
public:
   plot_pars(void);
   plot_pars(const plot_pars& pp) = default;
   ~plot_pars(void) = default;
   
   /**  Use logarithmic bins in a histogram.
    *  \brief Logarithmic histogram bins,
    *  \param lb Set logbin parameter.
    */
   void logbin(bool lb);
  
   /**  Set the color to be used in 1D (histogram, time series, etc.) plots.
    *  \brief Set the trace color.
    *  \param trace_col as defined by root color map.
    */
   void set_color(int trace_col);

   /**  Set the color to be used in 1D (histogram, time series, etc.) plots. 
    *  Index list is in the following table:
    *  <table>
    *  <tr><td>0</td><td>Black</td></tr> 
    *  <tr><td>1</td><td>Blue</td></tr> 
    *  <tr><td>2</td><td>Red</td></tr> 
    *  <tr><td>3</td><td>Green</td></tr> 
    *  <tr><td>4</td><td>Yellow</td></tr> 
    *  <tr><td>5</td><td>Magenta</td></tr> 
    *  <tr><td>6</td><td>Cyan</td></tr> 
    *  <tr><td>7</td><td>Orange</td></tr> 
    *  <tr><td>8</td><td>Spring</td></tr> 
    *  <tr><td>9</td><td>Teal</td></tr> 
    *  <tr><td>10</td><td>Azure</td></tr> 
    *  <tr><td>11</td><td>Violet</td></tr> 
    *  <tr><td>12</td><td>Pink</td></tr> 
    *  </table>
    *  \brief Set trace color by index number.
    *  \param color_index as defined by root color map.
    */
   void set_color_index(int color_index);

   /** set hte number of histogram bins.
    */
   void set_nbins(int n);

   
   /**  Set the parameters n the specified GDSPlot instance to the 
    *   pre-defined values.
    *   \param plot gdsplot isntance to receive te values.
    */
   void set_palette(const std::string& palette);
   
   /**  Set the parameters n the specified GDSPlot instance to the 
    *   pre-defined values.
    *   \param plot gdsplot isntance to receive te values.
    */
   void set_parameters(GDSPlot& plot) const;
   
   /**  Set x, and y ranges for following plot(s).
    *  \brief Set x and y range.
    *  \param xmin Minimum x value.
    *  \param xmax Maximum x value.
    *  \param ymin Minimum y value.
    *  \param ymax Maximum y value.
    */
   void set_range(double xmin, double xmax, double ymin, double ymax);

   /**  Set the smoothing. the \a nSmooth argument indicates the number 
    *  of smoothing passes are to be performed. Setting this argument 
    *  zero disables smoothing. The \a smoothOpt argument soecifies the 
    *  kernel to be used. 2D kernels include "k5a" (five points in each
    *  direction, default), "k5b" stronger than k5a, and "k3a".
    *  \brief Set smoothing parameters.
    *  \param nSmooth Number of smoothing passes or zero.
    *  \param smoothOpt Smoothing kernel.
    */
   void set_smooth(int nSmooth, const std::string& smoothOpt);

   /**  Set the statistics box status. If true, a histogram-style statistics 
    *  box is printed. If false the box will not be printed. By defaule the 
    *  statistics box flag is in an nundefined state and will be set 
    *  according to the style.
    *  \brief Set the statistics-box flag.
    *  \param tf If true, a statistics box will be plotted.
    */
   void set_stat_box(bool tf=true);

   /**  Set the graphics style. The default style (0) sets pads to flat white,
    *  disables statistics boxes, etc. Style (1) is used for plots with 
    *  z-axis color-bar and leaves room on the right for the z-axis title. 
    *  This function is used in the constructor.
    *  \brief Set the graphics style.
    *  \param id Style identifier.
    */
   void set_style(int id=0);

   /**  Set the plot size in.
    *  \brief set plot size.
    *  \param xsize X dimension in cm.
    *  \param ysize Y dimension in cm.
    */
   void set_size(double xsize, double ysize);

   /**  Set the trace name used in the legend.
    *  \brief Set trace name.
    *  \param name Trace name to be used in the legend;
    */
   void set_trace_name(const std::string& name);

   /**  Set the z range (color dimension) for x-y plots.
    *  \brief Set the z (color) dimension range.
    *  \param zmin Minimum z value.
    *  \param zmax Maximum z value.
    */
   void set_zrange(double zmin, double zmax);

   /**  Print the current status of the plot parameter set.
     *  \brief Print parameter status,
     *  \param out Output stream to which the parameters will be printed
     */
   void status(std::ostream& out) const;
   
   /**  Set the plot title string.
    *  \brief Plot title string.
    *  \param titl Title string.
    */
   void title(const std::string& titl);

   /**  Set the x-axis label string.
    *  \brief X-axis label.
    *  \param titl Text to be plotted below x axis.
    */
   void xlabel(const std::string& titl);

   /**  Set the x axis scale to logarithmic/linear
    *  \brief Set logarithmic x scale.
    *  \param logx set to true for logarithmic or false for linear scale. 
    */
   void xlog(bool logx=true);

   /**  Treat the x axis as a time. Set the units as appropriate to avoid 
    *  excessively large or small numbers.
    *  \brief Set a time axis.
    *  \param dt Time increment.
    *  \param xttl x title to be used.
    *  \return Multiplicative factor used to convert to selected time scale
    */
   double xTimeScale(double dt, const std::string& xttl="Time");
    
   /**  Set the y-axis label string.
    *  \brief Y-axis label.
    *  \param titl Text to be plotted to the left of the y axis.
    */
   void ylabel(const std::string& titl);

   /**  Set the y axis scale to logarithmic/linear
    *  \brief Set logarithmic y scale.
    *  \param logy set to true for logarithmic or false for linear scale. 
    */
   void ylog(bool logy=true);

   /**  Set the z-axis label string.
    *  \brief Z-axis label.
    *  \param titl Text to be plotted to the right of the Z axis color bar.
    */
   void zlabel(const std::string& titl);

   /**  Set the z axis scale to logarithmic/linear
    *  \brief Set logarithmic z scale.
    *  \param logz set to true for logarithmic or false for linear scale. 
    */
   void zlog(bool logz=true);

private:
   int    mStyleId;
   double mXsize;
   double mYsize;
   double mXmin;
   double mXmax;
   double mYmin;
   double mYmax;
   double mZmin;
   double mZmax;
   tri_state mLogX;
   tri_state mLogY;
   tri_state mLogZ;
   int       mNBin;
   tri_state mLogBin;
   tri_state mStatBox;
   int    mNextColor;
   int    mNSmooth;
   std::string mPalet;
   std::string mTitle;
   std::string mTraceName;
   std::string mXlabel;
   std::string mYlabel;
   std::string mZlabel;
   std::string mSmoothOpt;
   double      mTimeScale;
};

#endif  // !defined(GDSPLOT_PARS_HH)
